@@ -11,7 +11,7 @@ use constant DEBUG => $ENV{MICRO_TEMPLATE_DEBUG} || 0;
 
 use Carp 'croak';
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(encoded_string build_mt render_mt);
 our %EXPORT_TAGS = (
@@ -31,11 +31,22 @@ sub new {
         tag_start           => '<?',
         tag_end             => '?>',
         escape_func         => 'Text::MicroTemplate::escape_html',
+        package_name        => undef, # defaults to caller
         @_ == 1 ? ref($_[0]) ? %{$_[0]} : (template => $_[0]) : @_,
     }, $class;
     if (defined $self->{template}) {
         $self->parse($self->{template});
     }
+    unless (defined $self->{package_name}) {
+        $self->{package_name} = 'main';
+        my $i = 0;
+        while (my $c = caller(++$i)) {
+            if ($c ne __PACKAGE__) {
+                $self->{package_name} = $c;
+                last;
+            }
+        }
+    }
     $self;
 }
 
@@ -47,6 +58,14 @@ sub escape_func {
     $self->{escape_func};
 }
 
+sub package_name {
+    my $self = shift;
+    if (@_) {
+        $self->{package_name} = shift;
+    }
+    $self->{package_name};
+}
+
 sub template { shift->{template} }
 
 sub code {
@@ -265,7 +284,7 @@ sub parse {
         }
         push @{$self->{tree}}, \@token;
     }
-
+    
     return $self;
 }
 
@@ -282,26 +301,13 @@ sub _context {
 sub _error {
     my ($self, $error, $line_offset, $from) = @_;
     
-    unless (defined $from) {
-        $from = sub {
-            my $i = 1;
-            while (my @c = caller($i++)) {
-                if ($c[0] ne __PACKAGE__) {
-                    return "$c[1] at line $c[2]";
-                }
-            }
-            '';
-        }->();
-    }
-    $from = ' passed from ' . $from;
-    
     # Line
     if ($error =~ /^(.*)\s+at\s+\(eval\s+\d+\)\s+line\s+(\d+)/) {
         my $reason = $1;
         my $line   = $2 - $line_offset;
         my $delim  = '-' x 76;
         
-        my $report = "$reason at line $line in template$from.\n";
+        my $report = "$reason at line $line in template passed from $from.\n";
         my $template = $self->_context($self->{template}, $line);
         $report .= "$delim\n$template$delim\n";
 
@@ -309,9 +315,9 @@ sub _error {
         if (DEBUG) {
             my $code = $self->_context($self->code, $line);
             $report .= "$code$delim\n";
+            $report .= $error;
         }
 
-        $report .= $error;
         return $report;
     }
 
@@ -339,12 +345,21 @@ sub escape_html {
 sub build_mt {
     my $_mt = Text::MicroTemplate->new(@_);
     my $_code = $_mt->code;
+    my $_from = sub {
+        my $i = 0;
+        while (my @c = caller(++$i)) {
+            return "$c[1] at line $c[2]"
+                if $c[0] ne __PACKAGE__;
+        }
+        '';
+    }->();
     my $expr = << "...";
+package $_mt->{package_name};
 sub {
-    my \$args = \@_ == 1 ? \$_[0] : { \@_ };
-    encoded_string((
+    local \$SIG{__WARN__} = sub { print STDERR \$_mt->_error(shift, 4, \$_from) };
+    Text::MicroTemplate::encoded_string((
         $_code
-    )->());
+    )->(\@_));
 }
 ...
     my $die_msg;
@@ -353,7 +368,7 @@ sub {
         if (my $_builder = eval($expr)) {
             return $_builder;
         }
-        $die_msg = $_mt->_error($@, 3);
+        $die_msg = $_mt->_error($@, 4, $_from);
     }
     die $die_msg;
 }
@@ -389,15 +404,12 @@ Text::MicroTemplate
 
     use Text::MicroTemplate qw(:all);
 
-    # simple form
-    $html = render_mt(
-         'hello, <?= $args->{user} ?>',
-         { user => 'John' },
-    )->as_string;
+    # compile template, and render
+    $renderer = build_mt('hello, <?= $_[0] ?>');
+    $html = $renderer->('John')->as_string;
 
-    # cache compiled template as subref
-    $renderer = build_mt('hello, <?= $args->{user} ?>');
-    $html = $renderer->({ user => 'John' })->as_string;
+    # or in one line
+    $html = render_mt('hello, <?= $_[0] ?>', 'John')->as_string;
 
     # complex form
     $mt = Text::MicroTemplate->new(
@@ -459,42 +471,36 @@ Text::MicroTemplate does not provide features like template cache or including o
 
 =head1 EXPORTABLE FUNCTIONS
 
-=head2 render_mt($template, $args)
-
-Utility function that renders given template and returns an EncodedString.
-
-    # render
-    $hello = render_mt(
-        'hello, <?= $args->{user} ?>!',
-        { user => 'John' },
-    );
-
-    # print as HTML
-    print $hello->as_string;
-
-    # use the result in another template (no double-escapes)
-    $enc = render_mt(
-        '<h1><?= $args->{hello} ?></h1>',
-        { hello => $hello },
-    );
-
-Intertally, the function is equivalent to:
-
-    build_mt($template)->($args);
-
 =head2 build_mt($template)
 
 Returns a subref that renders given template.  Parameters are equivalent to Text::MicroTemplate->new.
 
     # build template renderer at startup time and use it multiple times
-    my $renderer = build_mt('hello, <?= $args->{user} ?>!');
+    my $renderer = build_mt('hello, <?= $_[0] ?>!');
 
     sub run {
         ...
-        my $hello = $renderer->({ user => $query->param('user') });
+        my $hello = $renderer->($query->param('user'));
         ...
     }
 
+=head2 render_mt($template, $args)
+
+Utility function that combines build_mt and call to the generated template builder.
+
+    # render
+    $hello = render_mt('hello, <?= $_[0] ?>!', 'John');
+
+    # print as HTML
+    print $hello->as_string;
+
+    # use the result in another template (no double-escapes)
+    $enc = render_mt('<h1><?= $_[0] ?></h1>', $hello);
+
+Intertally, the function is equivalent to:
+
+    build_mt($template)->(@_);
+
 =head2 encoded_string($str)
 
 wraps given string to an object that will not be escaped by the template engine
@@ -505,9 +511,23 @@ Text::MicroTemplate provides OO-style interface to handle more complex cases.
 
 =head2 new($template)
 
-=head2 new({ template => $template, escape_func => 'escape_funcname' })
+=head2 new(%args)
 
-In the first form, constructs a HTML template renderer.  In the second form, constructs a renderer that would escape non-encoded strings using given escape function.  If escape function is set to undef, no escape will occur.  The default escape function is C<Text::MicroTemplate::escape_html>.
+=head2 new(\%args)
+
+Constructs template renderer.  In the second or third form, parameters below are recognized.
+
+=head3 template
+
+template string (mandatory)
+
+=head3 escape_func
+
+escape function (defaults to C<Text::MicroTemplate::escape_html), no escape when set to undef
+
+=head3 package_name
+
+package under where the renderer is compiled (defaults to caller package)
 
 =head2 code()
 
